/**
 * 
 */
package org.testcase.issue7249.servlet;

import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.sql.DataSource;
import javax.transaction.UserTransaction;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.testcase.issue7249.exceptions.TechnicalException;

/**
 * @author e323296
 *
 */
public class TestServlet extends HttpServlet {

	/**
	 * 
	 */
	private static final long serialVersionUID = 239797250861048720L;
	private Log logger = LogFactory.getLog(TestServlet.class);

	private DataSource myNonXADatasource;
	private DataSource myXADatasource;

	/**
	 * 
	 */
	public void cleanDatasources() {
		myNonXADatasource = null;
		myXADatasource = null;
	}

	/**
	 * 
	 */
	public void initDatasources() {
		Context ctx;
		try {
			ctx = new InitialContext();
		} catch (NamingException e) {
			logger.error("Naming Exception.", e);
			throw new TechnicalException( "Naming Exception.", e );
		}

		try {
			myNonXADatasource = (DataSource)ctx.lookup("java:comp/env/jdbc/Datasource_NonXA1");
		} catch (NamingException e) {
			cleanDatasources();
			logger.error("Naming Exception when binding the Shareable non XA 1 datasource.", e);
			throw new TechnicalException( "Naming Exception when binding the Shareable non XA 1 datasource.", e );
		}

		try {
			myXADatasource = (DataSource)ctx.lookup("java:comp/env/jdbc/Datasource_XA");
		} catch (NamingException e) {
			cleanDatasources();
			logger.error("Naming Exception when binding the XA datasource.", e);
			throw new TechnicalException( "Naming Exception when binding the XA datasource.", e );
		}

	}


	/* (non-Javadoc)
	 * @see javax.servlet.http.HttpServlet#doGet(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
	 */
	@Override
	protected void doGet(HttpServletRequest req, HttpServletResponse resp)
	throws ServletException, IOException {

		logger.info("TestServlet - Get request ...");

		HttpSession session = req.getSession();

		try {
			initDatasources();
		} catch (TechnicalException te) {
			session.setAttribute("exception", te.getMessage());
			resp.sendRedirect(req.getContextPath() + "/ko.jsp");
			return;
		}

		String whichTest = (String) req.getParameter("test");
		if( whichTest == null || "".equals(whichTest) ) {
			logger.error("No parameter test defined !");
			session.setAttribute("exception", "No parameter test defined !");
			resp.sendRedirect(req.getContextPath() + "/ko.jsp");
			return;
		}

		boolean result;
		if( "2".equals(whichTest)) {
			result = testUseCase_2(req);
		} else {
			logger.error("Invalid value for the parameter test found !");
			result = false;
		}

		if( result ) {
			resp.sendRedirect(req.getContextPath() + "/ok.jsp");
		} else {
			resp.sendRedirect(req.getContextPath() + "/ko.jsp");
		}

	}

	/**
	 * Use case: One shareable XA connection then one shareable nonXA connection with UTX
	 * 
	 * @param req
	 * @return True -> OK / False -> KO
	 * @throws IOException
	 */
	private boolean testUseCase_2(HttpServletRequest req) throws IOException {

		logger.info("Use case: One shareable XA connection then one shareable nonXA connection with UTX");

		HttpSession session = req.getSession();
		UserTransaction utx = userTransactionLookUp();
		if( utx == null ) {
			session.setAttribute("exception", "userTransactionLookUp() failed ...");
			return false;
		}

		boolean toCommit = false;
		boolean responseDefined = false;

		try {
			utx.begin();

			Connection conn1 = null;
			try {
				conn1 = myXADatasource.getConnection();
				logger.info("myXADatasource.getConnection() OK ...");
			} catch (SQLException e) {
				cleanDatasources();
				logger.error("myXADatasource.getConnection() KO ...\n"+e.getMessage(), e);
				session.setAttribute("exception", e.getMessage());
				return false;
			}

			Connection conn2 = null;
			try {
				conn2 = myNonXADatasource.getConnection();
				logger.info("myNonXADatasource.getConnection() OK ...");
			} catch (SQLException e) {
				cleanDatasources();
				logger.error("myNonXADatasource.getConnection() KO ...\n"+e.getMessage(), e);
				session.setAttribute("exception", e.getMessage());
				responseDefined = true;
			}

			testConnection("myXADatasource", conn1);
			testConnection("myNonXADatasource", conn2);

			if( conn1 != null ) {
				try {
					conn1.close();
					logger.info("myXADatasource.close() OK ...\n");
				} catch (SQLException e) {
					cleanDatasources();
					logger.error("myXADatasource.close() KO ...\n"+e.getMessage(), e);
					if( !responseDefined ) {
						session.setAttribute("exception", e.getMessage());
					}
					responseDefined = true;
				}
			}

			if( conn2 != null ) {
				try {
					conn2.close();
					logger.info("myNonXADatasource.close() OK ...\n");
				} catch (SQLException e) {
					cleanDatasources();
					logger.error("myNonXADatasource.close() KO ...\n"+e.getMessage(), e);
					if( !responseDefined ) {
						session.setAttribute("exception", e.getMessage());
					}
					responseDefined = true;
				}
			}

			toCommit = true;

		} catch (Exception e) {
			cleanDatasources();
			logger.error("Exception raised ...\n"+e.getMessage(), e);
			if( !responseDefined ) {
				session.setAttribute("exception", e.getMessage());
			}
			responseDefined = true;
		} finally {
			if( toCommit ) {
				try {
					utx.commit();
					logger.info("utx.commit() OK ...");
				} catch (Exception e) {
					cleanDatasources();
					logger.error("utx.commit() KO...\n"+e.getMessage(), e);
					if( !responseDefined ) {
						session.setAttribute("exception", e.getMessage());
					}
					responseDefined = true;
				}
			} else {
				try {
					utx.rollback();
					logger.info("utx.rollback() OK ...");
				} catch (Exception e) {
					cleanDatasources();
					logger.error("utx.rollback() KO...\n"+e.getMessage(), e);
					if( !responseDefined ) {
						session.setAttribute("exception", e.getMessage());
					}
					responseDefined = true;
				}
			}
		}

		// No response defined => All is ok
		//
		if( !responseDefined ) {
			session.setAttribute("exception", "");
			return true;
		} else {
			return false;
		}
	}

	/**
	 * @return
	 * @throws NamingException 
	 */
	private UserTransaction userTransactionLookUp() {
		Context ctx;
		UserTransaction utx = null;
		try {
			ctx = new InitialContext();
			utx =  (UserTransaction)ctx.lookup("java:comp/UserTransaction");
			logger.info("UserTransaction lookup OK !");
		} catch (NamingException e) {
			logger.error("Naming Exception when binding the XA datasource.", e);
			return null;
		}

		return utx;
	}

	/**
	 * @param connStr TODO
	 * @param conn
	 * @throws Exception 
	 * @throws SQLException 
	 */
	public void testConnection( String connStr, Connection conn ) {

		Statement stat = null;
		ResultSet rs = null;

		try {
			try {
				stat = conn.createStatement();
			} catch (SQLException e) {
				logger.error( "Connection "+connStr+" : Error when creating the statement:\n", e );
				throw e;
			}

			try {
				rs = stat.executeQuery("select 1, 1 from dual");
			} catch (SQLException e) {
				logger.error( "Connection "+connStr+" : Error when executing the statement:\n", e );
				throw e;
			}

			int count = 0;
			try {
				while( rs.next() ) {
					count += rs.getInt(1);
				}
			} catch (SQLException e) {
				logger.error( "Connection "+connStr+" : Error while browsing the result set :\n", e );
			}

			logger.info("Connection "+connStr+" : Test OK");
		} catch ( Exception e ) {
			// On ignore l'exception : elle a dj t traite
			//
		} finally {

			if( rs != null ) {

				try {
					rs.clearWarnings();
				} catch (SQLException e) {
					logger.error( "Connection "+connStr+" : Error when clearing the result set:\n", e );
				}

				try {
					rs.close();
				} catch (SQLException e) {
					logger.error( "Connection "+connStr+" : Error when clearing the result set:\n", e );
				}
			}

			if( stat != null ) {
				try {
					stat.close();
				} catch (SQLException e) {
					logger.error( "Connection "+connStr+" : Error when closing the statement:\n", e );
				}
			}
		}
	}

}
